/*
 * ouvej, determines what, when and where the next lesson will be
 * Copyright (C) 2003 Olivier Mehani <shtrom@ssji.net, olmehani@etu.utc.fr>
 * 
 * $Id: cache.c,v 1.8 2003/12/06 23:56:29 shtrom Exp $
 * binary cache handling function to avoir reparsing the timetable when not
 * necessary
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <errno.h>
#include "ouvej.h"

#define TTFNAME		"/timetable"
#define CACHENAME	"/.ouvej.cache"

int cache_check(char *homedir)
{
	char *ttpath, *cachepath;
	struct stat ttstat, cachestat;

	ttpath=timetable_path(homedir);
	cachepath=cache_path(homedir);

	if(stat(ttpath,&ttstat)<0) {
		ERROR("can't open %s (%s)\n", ttpath, strerror(errno));
		exit(1);
	}
	
	if(stat(cachepath,&cachestat)==0) {

		DPRINT("%s: %d\n",ttpath,(int)ttstat.st_mtime);
		DPRINT("%s: %d\n",cachepath,(int)cachestat.st_mtime);

		if((int)ttstat.st_mtime>(int)cachestat.st_mtime) {
			fprintf(stderr,"Timetable is newer than cache, regenerating...\n");
			cache_regen(ttpath, cachepath);
		}
	} else {
		fprintf(stderr,"Cachefile not found, regenerating...\n");
		cache_regen(ttpath, cachepath);
	}
	return 0;
}

int cache_regen(char *ttpath, char *cachepath)
{
	ll_t *list;
	bt_t *btree;

	list=gen_list(ttpath);
	
	btree=btree_make(list);
	btree_write(btree, cachepath);

	return 0;
}

char *timetable_path(char *homedir)
{
	static char *ttp;

	/*
	 * the path is only generated the first time it's needed
	 */
	if(ttp)
		return ttp;

	ttp=malloc(strlen(homedir)+sizeof(TTFNAME)+1);
	if(!ttp) {
		ERROR("couldn't allocate memory\n");
		exit(1);
	}

	*ttp=0;
	strcat(ttp,homedir);
	strcat(ttp,TTFNAME);

	DPRINT("timetable is %s\n",ttp);

	return ttp;
}

char *cache_path(char *homedir)
{
	static char *cp;

	/*
	 * the path is only generated the first time it's needed
	 */
	if(cp)
		return cp;

	cp=malloc(strlen(homedir)+sizeof(CACHENAME)+1);
	if(!cp) {
		ERROR("couldn't allocate memory\n");
		exit(1);
	}

	*cp=0;
	strcat(cp,homedir);
	strcat(cp,CACHENAME);

	DPRINT("cache is %s\n",cp);

	return cp;
}
