/*
 * ouvej, determines what, when and where the next lesson will be
 * Copyright (C) 2003 Olivier Mehani <shtrom@ssji.net, olmehani@etu.utc.fr>
 * 
 * $Id: lesson.c,v 1.16 2003/12/08 19:00:56 shtrom Exp $
 * timetable parsing and lesson handling functions 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <time.h>
#include "ouvej.h"

ll_t *gen_list(char *filename)
{
	FILE *utt;
	char buffer[MAXLINESIZE];
	int rl_status, lnum=0, last_parse_status=0;
	ll_t *list=NULL, *cur=NULL;

	DPRINT("start\n");

	utt=fopen(filename,"r");
	if(!utt) {
		ERROR("can't open %s\n", filename);
		return NULL;
	}

	while((rl_status=read_line(utt,buffer))>=0){
		if(!rl_status) {
			lnum++;
			DPRINT("new line: %s\n",buffer);
			
//			DPRINT("status: current=%p next=%p\n",cur,cur?cur->next:NULL);
			
			if(!cur) {
				DPRINT("new list\n");
				cur=list=malloc(sizeof(ll_t));
				cur->next=NULL;
			} else if(last_parse_status==0) {
				cur->next=malloc(sizeof(ll_t));
				cur=cur->next;
				cur->next=NULL;
			} else
				last_parse_status=0;
			
			cur->lesson=gen_lesson(buffer);
			
			if(*cur->lesson.name==0) {
				ERROR("can't parse line %d\n",lnum);
				last_parse_status=1;
			}
		} else if(rl_status==RL_AGAIN)
			ERROR("line to long (%s...)\n",buffer);
		else
			DPRINT("useless line\n");
	}
	DPRINT("EOF\n");

	fclose(utt);

	return list;
}

int read_line(FILE *utt, char *buffer)
{
	int len=0, tlen=0;

	do {
		len+=tlen;
		if(!(tlen = fread(&buffer[len],1,1,utt)))
			return RL_EOF;
	} while(len<MAXLINESIZE && buffer[len]!='\n'); 

	if(len==0 || buffer[0]=='#' || only_spaces(buffer)) {
		return RL_USELESS;
	} else if(buffer[len]=='\n') {
		buffer[len]='\0';
		return RL_OK;
	}

	buffer[len]='\0';
	return RL_AGAIN;
}

lesson_t gen_lesson(char *b)
{
	lesson_t l;
	char *offsets[L_T_ITEMS];
	int i, j=0;
	int error=0;

	memset(offsets,0,L_T_ITEMS);

	for(i=0;i<L_T_ITEMS && b;i++,b=next_block(b))
		offsets[i]=b;

//	DPRINT("%d\n",i);

	if((i!=L_T_ITEMS) && (i!=L_T_ITEMS-1)) {
		ERROR("malformed line (%d elements)\n", i);
		error=1;
	}

	DPRINT("filling structure for %s %s\n",offsets[0],offsets[1]);

	/*
	 * now fill the structure from the processed string...
	 */
	if(lesson_set_name(&l,offsets[j++])) {
		ERROR("can't set name for %s %s\n",offsets[0], offsets[1]);
		error=1;
	}
	if(lesson_set_type(&l,offsets[j++])) {
		ERROR("can't set type for %s %s\n",offsets[0], offsets[1]);
		error=1;
	}
	/*
	 * if i is not L_T_ITEMS, we're sure i is L_T_ITEMS-1 or there is a 
	 * big problem
	 */
	if(i==L_T_ITEMS) {
		if(lesson_set_group(&l,offsets[j++])) {
			ERROR("cant set group for %s %s\n",offsets[0], offsets[1]);
			error=1;
		}
	} else {
		/*
		 * No error check here 'cause I _know_ the parameter is good
		 * I read the doc (what do you mean "Which doc ?" ?)
		 */
		lesson_set_group(&l,"-1");
	}
	if(lesson_set_day(&l,offsets[j++])) {
		ERROR("can't set day for %s %s\n",offsets[0], offsets[1]);
		error=1;
	}
	if(lesson_set_beg(&l,offsets[j++])) {
		ERROR("can't set beginning for %s %s\n",offsets[0], offsets[1]);
		error=1;
	}
	if(lesson_set_end(&l,offsets[j++])) {
		ERROR("can't set end for %s %s\n",offsets[0], offsets[1]);
		error=1;
	}
	if(lesson_set_seq(&l,offsets[j++])) {
		ERROR("can't set week sequence for %s %s\n",offsets[0], offsets[1]);
		error=1;
	}
	if(lesson_set_place(&l,offsets[j++])) {
		ERROR("can't set place for %s %s\n",offsets[0], offsets[1]);
		error=1;
	}

	if(error)
		*l.name=0; //we want to be able to notice an error happened

	return l;
}

char *next_block(char *b)
{
	char *p1=NULL, *p2=NULL;
	
	p1=strchr(b,' ');
	p2=strchr(b,'\t');

//	DPRINT("%p %p\n",p1,p2);

	if(!p1 && !p2)
		return NULL;
	else if(!p1 && p2)
		return trim(p2);
	else if(!p2 && p1)
		return trim(p1);

	return p1<p2?trim(p1):trim(p2);
}

char *trim(char *p)
{
	if(!p)
		return NULL;

	*p++=0;

	while((*p==' ' || *p=='\t')) {
		if(*p=='\0')
			return NULL;
		p++;
	}
	return p;
}

int only_spaces(char *b)
{
	DPRINT("only spaces in '%s' ?\n",b);
	while(b) {
		if(*b!=' ' && *b != '\t' && *b != '\r' && *b != '\n') {
			DPRINT("no\n");
			return 0;
		} else if (*b == '\r' || *b == '\n') {
			DPRINT("carriage return -> yes\n");
			return 1;
		}
		b++;
	}
	DPRINT("yes\n");
	return 1;
}

int lesson_set_name(lesson_t *l, char *name)
{
	if(!snprintf(l->name,L_NAME_LEN,"%s",name))
		return -1;
	return 0;
	
}

int lesson_set_type(lesson_t *l, char *type)
{
	
	l->type=type_from_string(type);
	if(l->type==L_ERROR)
		return -1;
	return 0;
}

int lesson_set_group(lesson_t *l, char *group)
{
	l->group=group_from_string(group);
	if(l->type<0)
		return -1;
	return 0;
}

int lesson_set_day(lesson_t *l, char *day)
{
	l->day=day_from_string(day);
	if(l->day==D_ERROR)
		return -1;
	return 0;
}

int lesson_set_seq(lesson_t *l, char *seq)
{
	l->week_seq=seq_from_string(seq);
	if(l->week_seq==W_ERROR)
		return -1;
	return 0;
}

int lesson_set_beg(lesson_t *l, char *beg)
{
	l->beginning=time_from_string(beg);
	if(l->beginning.hour<0 || l->beginning.minutes<0)
		return -1;
	return 0;
}

int lesson_set_end(lesson_t *l, char *end)
{
	l->end=time_from_string(end);
	if(l->end.hour<0 || l->end.minutes<0)
		return -1;
	return 0;
}

int lesson_set_place(lesson_t *l, char *place)
{
	if(!snprintf(l->place,L_PLACE_LEN,"%s",place))
		return -1;
	return 0;
}

lesson_type_t type_from_string(char *s)
{
	lesson_type_t lt=L_ERROR;

	switch(*s) {
	case 'C':
	case 'c':
		lt=L_C;
		break;
	case 'D':
	case 'd':
		lt=L_D;
		break;
	case 'T':
	case 't':
		lt=L_T;
		break;
	}
	
	return lt;
}

int group_from_string(char *s)
{
	int val;
	char* end;

	val=strtol(s,&end,10);
	if(end==s)
		return -1;
	
	return val;
}

week_day_t day_from_string(char *s)
{
	week_day_t wd=D_ERROR;

#define if_then(day,val)	if(!strncasecmp(s,day,3)) wd=val
/*
 * no, the semicolon is not missing after "wd=val"
 * it's just added in the following
 */
	if_then("LUN",D_MONDAY);
	else if_then("MAR",D_TUESDAY);
	else if_then("MER",D_WENESDAY);
	else if_then("JEU",D_THURSDAY);
	else if_then("VEN",D_FRIDAY);
	else if_then("SAM",D_SATURDAY);
	else if_then("DIM",D_SUNDAY);	//ok, it's useless :-)
#undef if_then	

	return wd;
}

week_sequence_t seq_from_string(char *s)
{
	week_sequence_t ws=W_ERROR;

	switch(*s) {
	case 'T':
	case 't':
		ws=W_BOTH;
		break;
	case 'A':
	case 'a':
		ws=W_A;
		break;
	case 'B':
	case 'b':
		ws=W_B;
		break;
	}
	return ws;
}

hour_t time_from_string(char *s)
{
	hour_t h={-1,0}; //unless otherwise stated, the minutes are 0
	char *end;

	/*
	 * BEWARE: only strings of the format HH:MM are expected
	 */

	h.hour=strtol(s,&end,10);
	if(end==s	//not good...
	   || h.hour<0
	   || h.hour>23)
		h.hour=-1;

	if(*end==':') { //if colon, we expect the next chunk to be the minutes
		s=++end; 
		h.minutes=strtol(s,&end,10);
		if(end==s
		   || h.minutes<0
		   || h.minutes>59)
			h.minutes=-1;
	}

	return h;
}

char *lesson_get_name(lesson_t *l)
{
	return l->name;
}

lesson_type_t lesson_get_type(lesson_t *l)
{
	return l->type;
}

int lesson_get_group(lesson_t *l)
{
	return l->group;
}

week_day_t lesson_get_day(lesson_t *l)
{
	return l->day;
}

week_sequence_t lesson_get_seq(lesson_t *l)
{
	return l->week_seq;
}

hour_t lesson_get_beg(lesson_t *l)
{
	return l->beginning;
}

hour_t lesson_get_end(lesson_t *l)
{
	return l->end;
}

char *lesson_get_place(lesson_t *l)
{
	return l->place;
}

int lesson_compare_beg(lesson_t l1, lesson_t l2)
{
	if(l1.day>l2.day)
		return 1;
	else if(l1.day<l2.day)
		return -1;
	else 
		return hour_compare(l1.beginning,l2.beginning);
}

/*
 * returns the difference in minutes 
 */
int lesson_compare_end(lesson_t l1, lesson_t l2)
{
	return (l1.day-l2.day)*24*60+hour_compare(l1.end,l2.end);
}

int hour_compare(hour_t t1, hour_t t2)
{
	return (t1.hour-t2.hour)*60+t1.minutes-t2.minutes;
}

void lesson_show(lesson_t l)
{
	printf("%s, type %d, group %d, day %d on weeks %d, from %02d:%02d to %02d:%02d, in %s\n",
		l.name,
		l.type,
		l.group,
		l.day,
		l.week_seq,
		l.beginning.hour,l.beginning.minutes,
		l.end.hour,l.end.minutes,
		l.place
		);
}

lesson_t lesson_from_time(time_t t)
{
	struct tm *ct;
	lesson_t l;

	memset(&l,0,sizeof(l));

	ct=localtime(&t);

	/*
	 * Gruuuuiiik
	 *
	 * cf ctime(3)
	 */
	l.day=(ct->tm_wday-1)%7;
	l.week_seq=W_BOTH;
	l.beginning.hour=l.end.hour=ct->tm_hour;
	l.beginning.minutes=l.end.minutes=ct->tm_min;

	return l;
}
