/*
 * ouvej, determines what, when and where the next lesson will be
 * Copyright (C) 2003 Olivier Mehani <shtrom@ssji.net, olmehani@etu.utc.fr>
 * 
 * $Id: user.c,v 1.5 2003/12/08 18:27:15 shtrom Exp $
 * pretty user output functions 
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#include <stdio.h>
#include "ouvej.h"

void user_clean_ansi()
{
	printf(ANSI_STRING,0);
}

void user_set_ansi(int fg, int bg, int style)
{
	user_clean_ansi();
	if(style>=0) printf(ANSI_STRING,style);
	if(fg>=0) printf(ANSI_STRING,FOREGROUND+fg);
	if(bg>=0) printf(ANSI_STRING,BACKGROUND+bg);
}

void user_show_current(lesson_t right_now)
{
	printf("Nous sommes %s, il est %s\n",user_day_name(right_now.day),user_time(right_now.end));
}

void user_show_lesson(lesson_t l, lesson_t right_now)
{
	int offset;

	if(!l.name[0]) {
		printf("Week End !\n");
	} else if (l.day>right_now.day) {
		printf("C'est fini pour aujourd'hui\n");
		printf("Prochain rendez-vous %s a %s, %s de %s en %s\n",
			user_day_name(l.day),
			user_time(l.beginning),
		 	user_lesson_type(l.type), 
			l.name,
			l.place);
	} else {
		printf("A %s, %s de %s en %s\n",
			user_time(l.beginning),
		 	user_lesson_type(l.type), 
			l.name,
			l.place);
		offset=hour_compare(l.beginning,right_now.beginning);
		if(offset<0) {
			printf("A la bourre de %d minute%c !\n",-offset,offset<1?'s':' ');
		} else if(offset>0) {
			printf("Dans %d minute%c...\n",offset,offset>1?'s':' ');
		} else {
			printf("Juste maintenant\n");
		}
	}
}

char *user_day_name(week_day_t d)
{
	char *dayname[]={"Lundi", "Mardi", "Mercredi", "Jeudi", "Vendredi", "Samedi","Dimanche", "ERREUR"};
	return dayname[d];
}

char *user_lesson_type(lesson_type_t lt)
{
	char *ltype[]={"cours","TD","TP","ERREUR"};
	return ltype[lt];
}

char *user_week_sequence(week_sequence_t seq)
{
	char *wseq[]={"Tout le temps","Semaine A","Semaine B","ERREUR"};
	return wseq[seq];
}

/*
 * WARNING: the returned string will only remain valid until the next call to
 * user_time()
 */
char *user_time(hour_t t)
{
	static char buffer[6];
	snprintf(buffer,6,"%02d:%02d",t.hour,t.minutes);
	return buffer;
}
